clear all, close all

load vessel_data.mat % It loads vectors u, y, h, f

Ni = length(u);
Ts = 60 * 4; % sampling time 240 sec, 4 minutes.
k_dec = 3;

ur = decimate(u,k_dec);
yr = decimate(y,k_dec);
fr = decimate(f,k_dec);
hr = decimate(h,k_dec);

%%% Detrend ???

Tsr = 60*4 * k_dec; % new sampling time

N = length(ur);
tb = linspace(0,N*Tsr,N);
nt = 60 * 60;  % Normalised time (hours)
tbn = tb/nt;

u1 = [ur fr hr];
y1 = yr;

figure, subplot(221), plot(tbn,y1,'-'), title('Pulp k-quality output'), xlabel('Time (hours)'), ylabel('y')
        subplot(222), plot(tbn,ur,'-'), title('Pulp k-quality input'), xlabel('Time (hours)'), ylabel('u')
        subplot(223), plot(tbn,fr,'-'), title('Pulp Input Flow'), xlabel('Time (hours)'), ylabel('f')
        subplot(224), plot(tbn,hr,'-'), title('Vessel Pulp Level'), xlabel('Time (hours)'), ylabel('h')
       
n_max = 20;
nk_max = 20;
lam_v = 0.98; % Forgetting factor
n_optim = 0;
nk_optim = 0;
lam_opt = 0;

lam_var = [0.95:0.01:1];

for lam=1:length(lam_var),
    for n=1:n_max,
        for nk=1:nk_max,
            [THM,y1hat] = rarx([y1 u1],[n [n n n] [nk nk nk]],'ff',lam_var(lam));
            Je(lam,n,nk) = (y1hat')*y1hat/N;
            if((n==1)&(nk==1)), Joptim = Je(lam,n,nk); end; 
            if(Je(lam,n,nk) < Joptim), Joptim = Je(lam,n,nk); n_optim = n; nk_optim = nk; lam_opt = lam_var(lam); end;
        end;
    end;
end;

[THM,y1hat_optim] = rarx([y1 u1],[n_optim [n_optim n_optim n_optim] [nk_optim nk_optim nk_optim]],'ff',lam_opt);

figure, plot(tbn,y1,'-',tbn,y1hat_optim,'--'), title('Measured (-) and estimated (--) pulp k-quality output'), 
        xlabel('Time (hours)'), ylabel('y(t)')
figure, plot(tbn,y1,'-',tbn,y1hat_optim,'--',tbn,y1-y1hat_optim,':'), 
        title('Measured (-), estimated (--) pulp k-quality output and (:) estimation error'), 
        xlabel('Time (hours)'), ylabel('y(t) and e(t)')
        
return
